-- Externe Tabelle
-- Directory erzeugen
create directory ext_dir as '/usr/apps/datafiles'; 
grant read, write on ext_dir to scott;

-- Externe Tabelle erzeugen
create_table emp_ext (
  employee_number CHAR(5), 
  employee_dob CHAR(20), 
  employee_last_name CHAR(20), 
  employee_first_name CHAR(15), 
  employee_middle_name CHAR(15), 
  employee_hire_date DATE) 
organization external (
  type ORACLE_LOADER 
  default directory ext_dir 
  access parameters (
    records delimited by newline 
	characterset AL32UTF8 
	badfile ext_dir: 'employee.bad' 
	logfile ext_dir: 'employee.log' 
	fields terminated by ";" (
	  employee_number CHAR(5), 
	  employee_dob CHAR(20), 
	  employee_last_name CHAR(18), 
	  employee_first_name CHAR(11), 
	  employee_middle_name CHAR(11), 
	  employee_hire_date CHAR(10) date_format DATE mask "mm/dd/yyyy"
	)
  )
  location ('employee.txt')
) reject limit unlimited;

-- Liste der Oracle-Zeichensatzkodierungen
select * 
  from v$nls_valid_values 
 where parameter = 'CHARACTERSET';

-- Einfgen von Daten aus der externen Tabelle
insert into emp (empno, ename, hiredate)
select to_number(employee_number), 
       employee_last_name,
	   employee_hire_date 
  from emp_ext 
 where employee_number not in ( 
       select to_char(empno)
	     from emp);

-- Externe Tabelle mit DATAPUMP
create table emp_ext 
organization external (
  type oracle_datapump 
  default directory ext_dir 
  location ('employee.dmp')
  ) 
as select * from emp;

-- Zugriff auf die exportierten Daten ber eine externe Tabelle
create table emp_import (
  empno number(4,0), 
  ename varchar2(10 byte), 
  jobvarchar2(9 byte), 
  mgr number(4,0), 
  hiredate date, 
  sal number(7,2), 
  comm number(7,2), 
  deptno number(2,0)
  ) 
organization external (
  type oracle_datapump 
  default directory ext_dir 
  location ('employee.dmp')
);

