-- Rekursive WITH-Klausel
with hierarchie (lvl, ename, empno) as (
     select 1 lvl, ename, empno
       from emp 
      where mgr is null
      union all 
     select h.lvl + 1, e.ename, e.empno
       from hierarchie h, emp e
      where h.empno = e.mgr) 
select lvl, ename
  from hierarchie;

-- Rekursive WITH-Klausel mit DEPTH-FIRST-Ansatz
with hierarchie (lvl, ename, empno) as (
     select 1 lvl, ename, empno
       from emp 
      where mgr is null
      union all 
     select h.lvl + 1, e.ename, e.empno
       from hierarchie h, emp e
      where h.empno = e.mgr) 
search depth first by ename set sort_seq
select lvl, ename, sort_seq
  from hierarchie
 order by sort_seq;
 
-- Rekursive WITH-Klausel mit DEPTH-FIRST-Ansatz
with hierarchie (lvl, ename, empno) as (
     select 1 lvl, ename, empno
       from emp 
      where mgr is null
      union all 
     select h.lvl + 1, e.ename, e.empno
       from hierarchie h, emp e
      where h.empno = e.mgr) 
search depth first by ename set sort_seq
select lvl, 
       lpad('.', (lvl - 1) * 2, '.') || ename name
  from hierarchie
 order by sort_seq;
 
-- Simulation der Funktionen CONNECT_BY_ROOT und SYS_CONNECT_BY_PATH
with hierarchie (lvl, ename, empno, pfad, root) as (
     select 1 lvl, ename, empno,
            '/' || ename pfad, ename root
       from emp 
      where mgr is null
      union all 
     select h.lvl + 1, e.ename, e.empno,
            h.pfad || '/' || e.ename, h.root
       from hierarchie h, emp e
      where h.empno = e.mgr) 
search depth first by ename set sort_seq
select lvl, ename, pfad, root
  from hierarchie
 order by sort_seq;
 
-- Simulation der Funktion IS_CYCLE
update emp
   set mgr = 7369
 where empno = 7839;
 
with hierarchie (lvl, ename, empno) as (
     select 1 lvl, ename, empno
       from emp 
      where empno = 7839
      union all 
     select h.lvl + 1, e.ename, e.empno
       from hierarchie h, emp e
      where h.empno = e.mgr) 
search depth first by ename set sort_seq
cycle ename set is_cycle to 1 default 0
select lvl, ename, sort_seq, is_cycle
  from hierarchie
 order by sort_seq;
 
rollback;

-- Simulation der Funktion IS_LEAF
with hierarchie (lvl, ename, empno) as (
     select 1 lvl, ename, empno
       from emp 
      where mgr is null
      union all 
     select h.lvl + 1, e.ename, e.empno
       from hierarchie h, emp e
      where h.empno = e.mgr) 
search depth first by ename set sort_seq
select lvl, ename, sort_seq,
       case when lvl - lead(lvl) over (order by sort_seq) < 0
       then 0
       else 1 end is_leaf
  from hierarchie
 order by sort_seq;
 
with hierarchie (lvl, ename, empno, is_leaf) as (
     select 1 lvl, ename, empno,
            (select decode(count(*), 0, 1, 0)
               from emp
              where mgr = m.empno) is_leaf
       from emp m
      where mgr is null
      union all 
     select h.lvl + 1, e.ename, e.empno,
            (select decode(count(*), 0, 1, 0)
               from emp
              where mgr = e.empno) is_leaf
       from hierarchie h, emp e
      where h.empno = e.mgr) 
search depth first by ename set sort_seq
select lvl, ename, is_leaf
  from hierarchie;
