﻿/* Skript zur Demonstration von Pivotierungen
 * Als Benutzer SCOTT ausführen
 */

/* Pivotierung mit Gruppenfunktionen */
-- Abfrage mit einer Gruppierung über JOB und DEPTNO
select deptno, job, sum(sal) sum_sal
  from emp
 group by deptno, job
 order by deptno, job;
 
-- Erster Schritt: Gehälter auf Abteilungen verteilen
select job,
       decode(deptno, 10, sal) dept_10,
       decode(deptno, 20, sal) dept_20,
       decode(deptno, 30, sal) dept_30
  from emp;
  
-- Summierung der Gehälter, gruppiert nach Beruf
select job,
       sum(decode(deptno, 10, sal)) dept_10,
       sum(decode(deptno, 20, sal)) dept_20,
       sum(decode(deptno, 30, sal)) dept_30
  from emp
 group by job;
 
/* Pivotierung mit PIVOT-Klausel */  
-- Eine View zur Vorbereitung der Daten
create or replace view emp_pivot as
select job, deptno, sal
  from emp;

-- Verwendung der PIVOT - IN-Klausel
select *
  from emp_pivot
 pivot (sum(sal) for deptno in 
         (10 as dept_10, 
          20 as dept_20, 
          30 as dept_30));

-- Filterung der pivot-Tabelle
select *
  from emp_pivot
 pivot (sum(sal) for deptno in 
         (10 as dept_10, 
          20 as dept_20, 
          30 as dept_30))
 where dept_30 < 3500;
 
-- Versuch der Darstellung der Arbeitsweise der pivot-Klausel
select *
  from (select job,
               sum(decode(deptno, 10, sal)) dept_10,
               sum(decode(deptno, 20, sal)) dept_20,
               sum(decode(deptno, 30, sal)) dept_30
          from emp_pivot
         group by job)
 where dept_30 < 3500;


-- Die gleiche Abfrage mit einem globalen Spaltenalias
select *
  from emp_pivot
 pivot (sum(sal) as sum_sal,
       for deptno in 
         (10 as dept_10, 
          20 as dept_20, 
          30 as dept_30));

-- Beispiel für die Verwendung mehrerer Gruppenfunktionen
select *
  from emp_pivot
 pivot (sum(sal) as sum_sal, 
        avg(sal) as avg_sal, 
        max(sal) as max_sal
       for deptno in 
         (10 as dept_10), 
          20 as dept_20, 
          30 as dept_30);
          
/* Pivotierung mit XML-Klausel */
-- Verwendung der Klausel ANY
select *
  from emp_pivot
 pivot xml
       (sum(sal) as sum_sal
        for deptno in (ANY));

-- Verwendung einer Unterabfrage zur Definition der Pivotspalten
select *
  from emp_pivot
 pivot xml
       (sum(sal) as sum_sal,
        avg(sal) as avg_sal
        for deptno in (
          select deptno
            from dept))
 where job in ('CLERK', 'MANAGER');
 
