-- Einfhrung in JSON
-- Tabelle mit Constraint zur Speicherung von JSON
create table json_test(
  id number, 
  json_instanz clob, 
  constraint pk_json_test primary key(id), 
  constraint chk_json_test_json check(json_instanz is json)
);

-- Abfrage des Datentyps JSON aus dem Data Dictionary
select * 
  from user_json_columns;

-- Einfgeoperation
insert into json_test 
values(1, q'~{ "PONumber":1600,"Reference":"ABULL-20140421","Requestor":"Alexi sBull","User":"ABULL","CostCenter":"A50","ShippingInstructions" :{"name":"AlexisBull","Address":{"street":"200SportingGreen","c ity":"SouthSanFrancisco","state":"CA","zipCode":99236,"country" :"UnitedStatesofAmerica"},"Phone":[{"type":"Office","number":"9 09-555-7307"},{"type":"Mobile","number":"415-5551234"}]},"Speci alInstructions":null,"AllowPartialShipment":false,"LineItems":[ {"ItemNumber":1,"Part":{"Description":"OneMagicChristmas","Unit Price":19.95,"UPCCode":13131092899},"Quantity":9.0},{"ItemNumbe r":2,"Part":{"Description":"LethalWeapon","UnitPrice":19.95,"UP CCode":85391628927},"Quantity":5.0}]}~');

commit;

-- Auslesen von JSON-Werten
select j.json_instanz.PONumber
  from json_test j;

select j.json_instanz.LineItems.Part.Description 
  from json_test j;

-- JSON-Funktionen
select j.json_instanz.ShippingInstructions.Phone 
  from json_test j 
 where json_exists(j.json_instanz, '$.ShippingInstructions.Phone');

-- Erzeugung eines INDEXES
create index idx_json_test_json on json_test(json_instanz) 
  indextype is ctxsys.context 
  parameters ( 'section group ctxsys.json_section_group sync (on commit)');

-- Verwendung des INDEXES
select j.json_instanz.ShippingInstructions.Address 
  from json_test j 
 where json_textcontains( j.json_instanz, '$.ShippingInstructions.Address.state', 'CA');

-- Funktion JSON_Query
select json_query('[0,1,2,3,4,5,6,7,8]', '$[0, 3 TO 5, 7]' with wrapper) value
  from dual;

-- Funktion JSON_TABLE
-- Ausgabe als JSON
select jt.phones 
  from json_test j, 
       json_table(
	     json_instanz, 
		 '$.ShippingInstructions' 
		 columns ( 
		   phones varchar2(100) format json path '$.Phone')
		 ) as jt;

-- Ausgabe als relationale Tabelle
select jt.* 
  from json_test, 
       json_table(
	     json_instanz, 
		 '$.ShippingInstructions.Phone[*]' 
		 columns ( 
		   row_number for ordinality, 
		   phone_type varchar2(10) path '$.type', 
		   phone_num varchar2(20) path '$.number')
		 ) as jt;

-- Entschachtelung hierarchischer Ebenen mit JSON_TABLE
select jt.* 
  from json_test, 
       json_table(
	     json_instanz, '$' 
		 columns ( 
		  requestor varchar2(32) path '$.Requestor', 
		  nested path '$.ShippingInstructions.Phone[*]' 
		    columns (
			  phone_type varchar2(32) path '$.type', 
			  phone_num varchar2(20) path '$.number'
			)
		  ) as jt;
		  
-- Funktion JSON_VALUE
select json_value('{"a":{"b":100}}', '$.a.b') value 
  from dual;

