/* nderungen an XML-Instanzen in SQL durchfhren */
-- Erster Schritt: Tabelle, die die vorweg erzeugten XML-Instanzen speichert
create table dept_emp_xml as
select deptno, 
         xmlelement("Mitarbeiterliste",
           xmlattributes(
             deptno "abteilung"
           ),
           xmlagg(
             xmlelement("Mitarbeiter",
               xmlattributes(
                 hiredate "einstellDatum",
                 empno "id"
               ),
               xmlforest(
                 initcap(ename) "Name",
                 initcap(job) "Beruf",
                 trim(to_char(sal, '999G990D00L')) "Gehalt"
               )
             )
           ) 
       ) liste
  from emp
 group by deptno;

select *
  from dept_emp_xml
 where deptno = 10;
  
update dept_emp_xml
   set liste = updatexml(
                 liste,
                 '/Mitarbeiterliste/Mitarbeiter[3]/Beruf/text()',
                 'PRSIDENT')
 where deptno = 10;
 
update dept_emp_xml
   set liste = updatexml(liste,
                 '//Mitarbeiter[3]/Beruf',
                 XMLType('<Beruf>PRSIDENT</Beruf>'),
                 '//Mitarbeiter[2]/Name',
                 XMLType('<Name>Mller</Name>'))
 where deptno = 10;
 
rollback;
 
-- Lschen eines Elementes
update dept_emp_xml
   set liste = deletexml(liste,
               '//Mitarbeiter[Name="King"]')
 where deptno = 10;
 
select *
  from dept_emp_xml
 where deptno = 10;
 
rollback;

-- Einfgen eines neuen Elementes
-- Ein Geschwisterkind einfgen
update dept_emp_xml
   set liste = 
       insertxmlbefore(liste,
         '//Mitarbeiter[2]',
         xmltype('<Mitarbeiter einstellDatum="12.03.2001">' ||
                 '  <Name>Meier</Name>' ||
                 '  <Beruf>Halbkreisingenieur</Beruf>' ||
                 '  <Gehalt>3.500,00</Gehalt>' ||
                 '</Mitarbeiter>'))
 where deptno = 10;
 
select liste
  from dept_emp_xml
 where deptno = 10;
 
rollback;
-- Ein Kindelement als letzes anfgen
update dept_emp_xml
   set liste = 
       appendChildXML(liste,
         '/Mitarbeiterliste',
         xmltype('<Mitarbeiter einstellDatum="12.03.2001">' ||
                 '  <Name>Meier</Name>' ||
                 '  <Beruf>Halbkreisingenieur</Beruf>' ||
                 '  <Gehalt>3.500,00</Gehalt>' ||
                 '</Mitarbeiter>'))
 where deptno = 10;

select liste
  from dept_emp_xml
 where deptno = 10;
 
rollback;
-- Ein Kindelement vor dem ersten Mitarbeiter einfgen
update dept_emp_xml
   set liste = 
       insertChildXMLbefore(liste,
         '/Mitarbeiterliste',
         'Mitarbeiter[1]',
         xmltype('<Mitarbeiter einstellDatum="2001-03-12">' ||
                 '  <Name>Meier</Name>' ||
                 '  <Beruf>Halbkreisingenieur</Beruf>' ||
                 '  <Gehalt>3.500,00</Gehalt>' ||
                 '</Mitarbeiter>'))
 where deptno = 10;

select liste
  from dept_emp_xml
 where deptno = 10;
 
rollback;

/* Entschachteln der XML-Instanz */
-- Erster Schritt:
select v.*
  from dept_emp_xml x,
       table(
         xmlsequence(
           extract(x.liste, '//Mitarbeiter'))) v
 where deptno = 10;
 
-- Nchster Schritt: Einen Namen extrahieren
select upper(extractValue(v.column_value, '//Name')) ename,
       upper(extractValue(v.column_value, '//Beruf')) job,
       to_date(
         extractValue(
           v.column_value, '/Mitarbeiter/@einstellDatum'),
         'yyyy-mm-dd') hiredate,
       to_number(
         extractValue(
           v.column_value, '//Gehalt'),
         '999G990D00L') sal,
       x.deptno
  from dept_emp_xml x,
       table(
         xmlsequence(
           extract(x.liste, '//Mitarbeiter'))) v;