-- Verwendung des Typs XMLTYPE

-- Erzeugung einer Tabelle mit XMLTYPE
create table xml_test_1( 
  id number, 
  xml_content XMLType);

-- Oder als Objektorientierte Tabelle:
create table xml_test_2 of xmltype;

desc xml_test_2;

-- Insert-Anweisung mit XMLType
insert into xml_test_2
values (XMLType('<Foo/>'));

select *
  from xml_test_2;
  
insert into xml_test_1
values (1, XMLType('<Foo/>'));

select *
  from xml_test_1;
  
-- Erzeugung einer etwas lngeren XML-Instanz zur anschlieenden Bearbeitung
insert into xml_test_1 
values (1, XMLType('<Mitarbeiter id="12345"><Name>Mller</Name></Mitarbeiter>'));

desc XMLType;

-- Umformung mittels XSLT
create or replace view emp_xml_vw as 
select xmlelement("Mitarbeiterliste",
         xmlagg(
		   xmlelement("Mitarbeiter", 
		     xmlattributes( 
			   empno "id",
			   hiredate "einstellDatum", 
			   deptno "abteilungId" 
			 ), 
			 xmlforest( 
			   initcap(ename) "Name", 
			   initcap(job) "Beruf", 
			   sal "Gehalt" 
			 ) 
		   ) order by ename
		 )
	   ) resultat 
  from emp; 

-- Tabelle zur Speicherung der XSLT-Instanz
create table emp_xslt( 
  id number, 
  xslt xmltype, 
  constraint pk_emp_xslt primary key(id) 
);

-- Einfgen einer XSLT-Instanz
insert into emp_xslt 
values (10, xmltype(q'{
<?xml version="1.0" encoding="UTF-8"?> 
  <xsl:stylesheet version="1.0" 5 xmlns:xsl="http://www.w3.org/1999/XSL/Transform"> 
    <xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes"/> 
	<xsl:template match="/">
	  <xsl:apply-templates/>
	</xsl:template>
	<xsl:template match="Mitarbeiterliste">
	  <ManagerListe> 
	    <xsl:apply-templates select="Mitarbeiter[Beruf = 'Manager']"/> 
      </ManagerListe>
	</xsl:template>
	<xsl:template match="Mitarbeiter">
      <xsl:copy-of select="."/>
	</xsl:template>
  </xsl:stylesheet>}'));
  
commit;

-- Umwandlung:
select e.resultat.transform(x.xslt) umwandlung
  from emp_xml_vw e
 cross join emp_xslt x 
 where x.id = 1;

-- Lesen und Schreiben von XMLType in das Dateisystem
-- Directory erzeugen
create directory xml_imp_dir as 'C:\temp';

-- XML lesen
select dbms_xslprocessor.read2clob('XML_IMP_DIR', 'test.xml') resultat 
  from dual;

