-- Einfache XQuery-Abfragen

-- Tabelle, die die vorweg erzeugten XML-Instanzen speichert
create table dept_emp_xml as
select deptno, 
         xmlelement("Mitarbeiterliste",
           xmlattributes(
             deptno "abteilung"
           ),
           xmlagg(
             xmlelement("Mitarbeiter",
               xmlattributes(
                 hiredate "einstellDatum",
                 empno "id"
               ),
               xmlforest(
                 initcap(ename) "Name",
                 initcap(job) "Beruf",
                 trim(to_char(sal, '999G990D00L')) "Gehalt"
               )
             )
           ) 
       ) liste
  from emp
 group by deptno;
 
-- Einfhrendes Beispiel
 select xmlquery(
          '/Mitarbeiterliste/Mitarbeiter' 
		  passing liste
		  returning content) ergebnis
  from dept_emp_xml;

-- XQuery-Abfrage mit FLWOR-Ausdruck
select xmlquery(
         'for $emp in fn:collection("oradb:/SCOTT/EMP")
		  where $emp/ROW/SAL >= 3000 
		  return <Mitarbeiter> 
		         <Name>{$emp/ROW/ENAME/text()}</Name> 
				 <Gehalt>{$emp/ROW/SAL/text()}</Gehalt> 
				 </Mitarbeiter>'
		 returning content) ergebnis
  from dual;

-- FLWOR-Ausdruck mit Join
select xmlquery(
         'for $emp in fn:collection("oradb:/SCOTT/EMP"), 
		      $dept in fn:collection("oradb:/SCOTT/DEPT") 
		  where $emp/ROW/DEPTNO = $dept/ROW/DEPTNO
		    and $emp/ROW/SAL >= 3000 
		  return <Mitarbeiter> 
		         <Name>{$emp/ROW/ENAME/text()}</Name> 
				 <Gehalt>{$emp/ROW/SAL/text()}</Gehalt> 
				 <Abteilung>{$dept/ROW/DNAME/text()} </Abteilung> 
				 </Mitarbeiter>' 
		 returning content) ergebnis 
  from dual;

-- Funktion XMLTABLE
-- Einfhrung: Lesen einer HTTP-Ressource
-- Vorbedingung: Leseberechtigung auf die Ressource. Seit Version 11g muss hierfr eine ACL eingerichtet werden
-- Vorgehen fr Version 11g:
begin
  begin
    dbms_network_acl_admin.drop_acl (
      acl          => 'ezb_access.xml');
  exception
    when others then null;
  end;
  dbms_network_acl_admin.create_acl (
    acl          => 'ezb_access.xml', 
    description  => 'Grant to access EZB resources',
    principal    => 'SCOTT',
    is_grant     => true, 
    privilege    => 'connect');
    
  dbms_network_acl_admin.assign_acl (
    acl          => 'ezb_access.xml',
    host         => 'www.ecb.europa.eu', 
    lower_port   => 80,
    upper_port   => 80); 
end;
/

-- Vorgehen fr Version 12c:
begin
  dbms_network_acl_admin.appen_host_ace(
    host        => 'www.www.ecb.europa.eu.de',
    lower_port  => 80,
    ace         => xs$ace_type(
                         privilege_list => xs$name_list('http'),
                         principal_name => 'SCOTT'
                         principal_type => xs_acl.ptype_db));
end;
/


-- Abfrage der Ressource
select httpUriType('http://www.ecb.europa.eu/stats/eurofxref/ eurofxref-daily.xml').getXml() datei 
  from dual;

-- Extraktion der Umrechungskurse
select currency, rate 
  from xmltable( 
         xmlnamespaces (default 'http://www.ecb/eurofxref'), 
		 '//Cube[@time]/Cube' 
		 passing (httpuritype('http://www.ecbdaily.xml').getxml()) 
		 columns 
		   currency char(3 byte) path '@currency', 
		   rate number path 'fn:translate(@rate, ".", ",")');

-- Funktion XMLEXISTS
select liste 
  from dept_emp_xml 
 where xmlexists(
         '/Mitarbeiterliste/Mitarbeiter[Name="King"]'
         passing liste); 
		 
-- Funktion XMLCAST
select xmlcast(
         xmlquery( 
		   'for $e in //Mitarbeiter 
		    where $e/Name = "Scott" 
			return fn:translate($e/Gehalt, ",.", ",")' 
		 passing liste 
		 returning content) as number) sal 
  from dept_emp_xml 
 where xmlexists(
         '/Mitarbeiterliste/Mitarbeiter[Name="Scott"]' 
		 passing liste);
		 

