-- Tabelle mit Beispieldaten
create table patient_mit_dekubitus(
  pat id number,
  pat_seit date,
  pat_bis date,
  dekubitus_seit date,
  constraint pk_pat_m_dek primary key(pat_id)
);

insert into patient_mit_dekubitus
select 123 pat_id,
       date '2015-02-01' pat_seit,
       date '2015-02-24' pat_bis,
       date '2015-02-04' dekubitus_seit
  from dual
union all
select 234, date '2015-02-02', date '2015-02-19', date '2015-02-03' from dual union all
select 345, date '2015-02-04', date '2015-02-28', date '2015-02-08' from dual union all
select 456, date '2015-02-05', date '2015-02-11', date '2015-02-15' from dual;

commit;

create table fotodokumentation(
  dok_id number,
  pat_id number,
  dok_am date,
  constraint pk_fotodok primary key(dok_id),
  constraint fk_pat_id foreign key(pat_id) references patient_mit_dekubitus(pat_id)
) organization index;

create index idx_fk_fotodok_pat on fotodokumentation(pat_id);

insert into fotodokumentation
select 1 dok_id,
       123 pat_id,
       date '2015-02-03' dok_am
  from dual
union all
select 2, 123, date '2015-02-09' from dual union all
select 3, 123, date '2015-02-12' from dual union all
select 4, 123, date '2015-02-18' from dual union all
select 5, 123, date '2015-02-23' from dual union all
select 6, 234, date '2015-02-03' from dual union all
select 7, 234, date '2015-02-18' from dual union all
select 8, 234, date '2015-02-19' from dual union all
select 9, 345, date '2015-02-08' from dual union all
select 10, 345, date '2015-02-15' from dual union all
select 11, 345, date '2015-02-18' from dual union all
select 12, 345, date '2015-02-25' from dual;

commit;

-- Abfrage, die klärt, wo gegen die Regel verstoßen wurde:
with pat as(
     select p.pat_id, p.pat_seit, p.pat_bis,
            p.dekubitus_seit dek_seit, f.dok_am
       from patient_mit_dekubitus p
       left join fotodokumentation f on p.pat_id = f.pat_id)
select pat_id, max(coalesce(match_nr, 0)) erfuellt
  from pat
 match_recognize(
       partition by pat_id
       order by dok_am
       measures
         match_number() as match_nr
       all rows per match with unmatched rows
       pattern (strt dok+ ende)
       define
         dok as dok_am <=
           prev(coalesce(dok_am, dek_seit)) + interval '7' day,
         ende as dok_am + interval '7' day >= pat_bis
       )
 group by pat_id;

-- Abfrage der einzelnen Intervalle und der Dauer
  with pat as(
       select p.pat_id, p.pat_seit, p.pat_bis, 
              p.dekubitus_seit dek_seit, f.dok_am
         from patient_mit_dekubitus p
         join fotodokumentation f on p.pat_id = f.pat_id)
select pat_id, dek_seit, dok_am, pat_bis,
       abstand, dok_bis_entlassung,
       coalesce(match_nr, 0) erfuellt
  from pat
 match_recognize(
  partition by pat_id
  order by dok_am
  measures
    match_number() as match_nr,
    case
    when ende.dok_am is not null then
      ende.dok_am - dok.dok_am
    when dok.dok_am is not null then
      dok.dok_am - coalesce(prev(dok.dok_am), strt.dek_seit)
    else
      strt.dok_am - strt.dek_seit
    end
    as abstand,
    strt.pat_bis - ende.dok_am as dok_bis_entlassung
  all rows per match with unmatched rows
  pattern (strt dok+ ende)
  define
    dok as dok_am <= 
      prev(coalesce(dok_am, dek_seit)) + interval '7' day,
    ende as dok_am + interval '7' day >= pat_bis
 );