
-- ABfrage verteilt Produkte nach Umsatz auf drei Ausgabegruppen
with prods as(
     select prod_id, sum(quantity_sold) quantity
       from sales
      group by prod_id)
select prod_group, gr1_sum, gr2_sum, gr3_sum, final_sum_overall
  from prods
 match_recognize(
       order by quantity
       measures
         classifier() as prod_group,
         sum(gr1.quantity) as gr1_sum,
         sum(gr2.quantity) as gr2_sum,
         sum(gr3.quantity) as gr3_sum,
         sum(quantity) as sum_overall,
         final sum(quantity) as final_sum_overall,
         final sum(gr1.quantity) as final_gr1_sum
       all rows per match
       pattern ((gr1|gr2|gr3)*)
       define
         gr1 as count(gr1.*) = 1
           or sum(gr1.quantity) - gr1.quantity
           <= least(sum(gr2.quantity), sum(gr3.quantity)),
        gr2 as count(gr2.*) = 1
          or sum(gr2.quantity) - gr2.quantity <= sum(gr3.quantity)
       )
 order by prod_group, sum_overall;

-- Alternative Implementierung mit analytischen Funktionen
with prods as(
     select prod_id, sum(quantity_sold) quantity,
            mod(row_number()
              over (order by sum(quantity_sold)), 3) + 1 prod_group
       from sales
      group by prod_id)
select prod_id, quantity, prod_group,
       sum(quantity)
         over (partition by prod_group order by quantity) gr_sum,
  from prods
 order by prod_group, quantity;
