-- Dikussion: Speicherung von Objekten in der Datenbank
create type emp_typ as object(
  empno number(4),
  ename varchar2(10),
  job varchar2(10),
  mgr number(4),
  hiredate date,
  sal number(7,2),
  comm number(7,2)
);
/

create type emp_tab_typ as table of emp_typ;
/

create table dept_obj (
  deptno number(2),
  dname varchar2(20),
  loc varchar2(20),
  employees emp_tab_typ,
  constraint dept_obj_pk primary key (deptno)
  )
  nested table employees store as emp_nt;

-- Vergeblich: Fremdschlssel sind auf NESTED TABLES nicht mglich
alter table emp_nt
  add constraint emp_nt_mgr_empno_fk
  foreign key (mgr) references emp_nt(empno);

-- Einfgen von Testdaten
insert into dept_obj
select deptno, dname, loc,
       cast(
         multiset(
           select empno, ename, job, 
                  mgr, hiredate, sal, comm
             from emp e
            where e.deptno = d.deptno)
         as emp_tab_typ
       )
  from dept d;

-- Darstellung der Objekte als relationale Daten:
select d.deptno, d.dname, d.loc,
       e.empno, e.ename, e.job
  from dept_obj d,
       table(d.employees) e;

-- Vergeblich: Update auf alle Mitarbeiter
update table(select employees
               from dept_obj) e
   set e.sal = e.sal * 1.05;
   
-- Stattdessen mssen alle Abteilungen einzeln aktualisiert werden:
update table(select employees
               from dept_obj
              where deptno = 10) e
   set e.sal = e.sal * 1.05;

-- Fehler, wenn eine nicht existente Abteilung angesprochen wird:
update table(select employees
               from dept_obj
              where deptno = 11) e
   set e.sal = e.sal * 1.05;
