-- Verwendung von Nested Tables.
-- Deklaration eines Objekts, das in einer Nested Table gespeichert werden soll
create type Kunde_Typ as object (
  KNr         integer,
  Name        varchar2(20),
  Adresse     Adresse_Typ,
  Tel_liste   Telefon_Array
  );
/

-- Verwendung dieses Objekts
select kunde_typ(
         10, 'Mller',
         adresse_typ(
           12345, 'Dorthausen', 'Hohlweg', '7'),
         telefon_array(
           '+49-1234-5678',
           '+49-2345-7896-15')) kunde
  from dual;

-- Zugriff auf eines Attribute
  with kunden as(
       select kunde_typ(
              10, 'Mller',
              adresse_typ(
                12345, 'Dorthausen', 'Hohlweg', '7'),
              telefon_array(
                '+49-1234-5678',
               '+49-2345-7896-15')) kunde
       from dual)
select k.kunde.adresse.ort
  from kunden k;

-- Erzeugen einer Neste Table
create type kunden_liste_tab as
table of kunde_typ;
/

-- Beispiel: Kundentabelle, als Benutzer HR ausfhren
-- Erstellung der Objekte
connect hr/hr;

create or replace type Adresse_Typ as object (
  Plz         varchar2(12),
  Ort         varchar2(40),
  Strasse     varchar2(40),
  Hausnr      varchar2(5)
  );
/

create or replace type telefon_array as
varray(3) of varchar2(20);
/

create type Kunde_Typ as object (
  KNr         integer,
  Name        varchar2(20),
  Adresse     Adresse_Typ,
  Tel_liste   Telefon_Array
  );
/

create or replace type kunden_liste_tab as
table of kunde_typ;
/

-- Erstellen einer objektorientierten Abfrage, Rohdaten
select e.employee_id knr,
       e.last_name name,
       l.postal_code plz,
       l.city ort,
       l.street_address strasse,
       null hausnr,
       e.phone_number telefon_1,
       e.email telefon_2
  from employees e
  join departments d on e.department_id = d.department_id
  join locations l on d.location_id = l.location_id;

-- Umwandlung der relationalen Daten in Objekte
with kunden as (
     select e.employee_id knr,
	 	    e.last_name name,
 		    l.postal_code plz, 
		    l.city ort,
		    l.street_address strasse,
		    null hausnr,
		    e.phone_number telefon_1,
		    e.email telefon_2
	   from employees e
	   join departments d on e.department_id = d.department_id
	   join locations l on d.location_id = l.location_id)
select kunde_typ(
         k.knr, k.name,
         adresse_typ(
           k.plz, k.ort, k.strasse, k.hausnr),
         telefon_array(
           k.telefon_1, k.telefon_2)) kunden
  from kunden k;

-- Umwandlung der Objekte in eine Nested Table
with kunden as(
     select e.employee_id knr,
	 	    e.last_name name,
 		    l.postal_code plz, 
		    l.city ort,
		    l.street_address strasse,
		    null hausnr,
		    e.phone_number telefon_1,
		    e.email telefon_2
	   from employees e
	   join departments d on e.department_id = d.department_id
	   join locations l on d.location_id = l.location_id)
select cast(
         multiset(
           select kunde_typ(
                  k.knr, k.name,
                  adresse_typ(
                    k.plz, k.ort, k.strasse, k.hausnr),
                  telefon_array(
                    k.telefon_1, k.telefon_2))
             from kunden k
         ) as kunden_liste_tab
       )
  from dual;
