-- Objektorientierung
-- Verwendung eines Objekts
create type adresse_typ as object(
  Plz number(5), 
  Ort varchar2(40),
  Strasse varchar2(40), 
  Hausnr varchar2(5) 
);
/

desc adresse_typ;

-- Verwendung der Konstruktormethode
select adresse_typ( 2 12345, 'Dorthausen', 'Hohlweg', '7') adresse 
  from dual;

-- Zugriff auf die Objektattribute
 with objekt as (
      select adresse_typ(12345, 'Dorthausen', 'Hohlweg', '7') adresse
  	    from dual) 
select o.adresse.plz plz, o.adresse.ort ort, o.adresse.strasse strasse, o.adresse.hausnr hausnr 
  from objekt o;

-- Fehler, wenn kein Tabellenalias zum Zugriff vereinbart wird:
 with objekt as (
      select adresse_typ(12345, 'Dorthausen', 'Hohlweg', '7') adresse
  	    from dual) 
select adresse.plz plz, adresse.ort ort, adresse.strasse strasse, adresse.hausnr hausnr 
  from objekt;

-- Objekt, um relationale Daten aufzunehmen
create type mitarbeiter_typ as object (
  emp_id number,
  emp_name varchar2(20 char), 
  jobvarchar2(20 char), 
  salary number, 
  hire_date date, 
  department_id number
);
/

-- Abfrage, um relationale Daten als Objekt darzustellen
create or replace view emp_obj as
select mitarbeiter_typ(empno, ename, job, sal, hiredate, deptno) mitarbeiter_typ
  from emp;
  
select mitarbeiter
  from emp_obj;

-- Objekt wird durch Konstruktormethode erweitert
create or replace type mitarbeiter_typ as object (
  emp_id number,
  emp_name varchar2(20 char),
  job varchar2(20 char),
  salary number,
  hire_date date,
  department_id number,
  constructor function mitarbeiter_typ(
    self in out nocopy mitarbeiter_typ,
    emp_name in varchar2)
    return self as result,
  constructor function mitarbeiter_typ(
    self in out nocopy mitarbeiter_typ,
    emp_name in varchar2,
    job in varchar2,
    salary in varchar2 default null,
    department in number default null)
    return self as result
);
/

-- Implementierung der Konstruktormethoden im Typkrper
create or replace
type body mitarbeiter_typ
as
  constructor function mitarbeiter_typ(
    self in out nocopy mitarbeiter_typ,
    emp_name in varchar2)
    return self as result
  as
  begin
    self.emp_id := emp_seq.nextval;
    self.emp_name := emp_name;
    self.hire_date := trunc(sysdate);
    return;
  end;

  constructor function mitarbeiter_typ(
    self in out nocopy mitarbeiter_typ,
    emp_name in varchar2,
    job in varchar2,
    salary in varchar2 default null,
    department in number default null)
    return self as result
  as
  begin
    self.emp_id := emp_seq.nextval;
    self.emp_name := emp_name;
    self.hire_date := trunc(sysdate);
    self.job := job;
    self.salary := salary;
    self.department_id := department;
    return;
  end;
end;
/

-- Verwendung der Konstruktormethoden
select mitarbeiter_typ('Mller') mitarbeiter
  from dual;

select mitarbeiter_typ(
         'Mller', 'Analyst', 1500) mitarbeiter
  from dual;

  
