-- Objektorientierte Tabellen
-- Anlegen einer OO-Tabellen
create table kunden of kunde_typ;

desc kunden;

-- Darstellung der OID
select k.sys_nc_oid$, dump(k.sys_nc_oid$)
  from kunden k
 where rownum = 1;

-- Interne Speicherung des Adresstyps
select dump(k.adresse)
  from kunden k
 where rownum = 1;

-- Besonderheiten bei der Speicherung von Nested Tables
create table akquisition(
  produkt_id number,
  status_id number,
  interessenten kunden_liste_tab,
  kunden kunden_liste_tab)
  nested table interessenten 
   store as interessenten_nt
  nested table kunden 
   store as kunden_nt;

-- Vergeblicher Versuch, auf eine Nested Table zuzugreifen
select *
  from kunden_nt;

-- Einfgen von Daten in Tabelle AKQUISITION
insert into akquisition
select 15, 20,
       cast(
         multiset(
           select k.*
             from kunden k
            where k.knr between 200 and 210)
         as kunden_liste_tab),
       cast(
         multiset(
           select k.*
             from kunden k
            where k.knr between 110 and 120)
         as kunden_liste_tab)
  from dual;

commit;

-- Zugriff auf die Objektattribute
select distinct i.adresse.ort
  from akquisition a,
       table(a.interessenten) i;

-- Referenzen auf andere Objekte
create table angebote(
  id number,
  kunde ref kunde_typ);

-- Einfgen eines Datensatzes mit Referenz auf ein Objektattribute
insert into angebote (id, kunde)
select 10, ref(k)
  from kunden k
 where k.knr = 200;
 
-- Darstellung der Speicherung
select *
  from angebote;
  
select dump(a.kunde)
  from angebote a;

-- Auflsen der Objektreferenz beim Abfragen der Werte
select id, a.kunde.name, a.kunde.adresse.ort
  from angebote a;


