-- Indizierung von Fremdschlüsselspalten
-- Primärschlüssel sind indiziert und werden genutzt
explain plan for
select e.ename, d.dname
  from emp e
  join dept d on e.deptno = d.deptno
 where e.empno = 7369;

 @?\rdbms\admin\utlxpls
 
-- Wird umgekehrt über Fremdschlüssel gesucht, ist dies nicht der Fall:
explain plan for
select d.dname, count(e.empno)
  from dept d
  join emp e on d.deptno = e.deptno
 group by d.dname;

@?\rdbms\admin\utlxpls

-- Wird der Fremdschlüssel indiziert, ist dieses Problem behoben:
create index idx_emp_deptno on emp(deptno);

explain plan for
select d.dname, count(e.empno)
  from dept d
  join emp e on d.deptno = e.deptno
 group by d.dname;

@?\rdbms\admin\utlxpls


-- Skript zum Erkennen nicht indizierter Fremdschlüsselspalten
 with constraint_cols as (
       select b.table_name, a.constr_name, 
              a.pos, a.col_name,
              count(*) over (
                partition by constr_name) col_cnt
         from (select substr(
                        table_name,1,30) table_name,
                      substr(
                        constraint_name,1,30) constr_name,
                      substr(
                        column_name,1,30) col_name,
                      position pos
                 from user_cons_columns ) a,
              user_constraints b
        where a.constr_name = b.constraint_name
          and b.constraint_type = 'R')
select table_name tabelle, constr_name,
       listagg(col_name, ', ') 
       within group (order by col_name) spalten
  from constraint_cols cons
 where col_cnt > all
         ( select count(*)
             from user_ind_columns i
            where i.table_name = cons.table_name
              and i.column_name = col_name
              and i.column_position <= cons.col_cnt
            group by i.index_name
         )
 group by table_name, constr_name;


-- Beispiel für einen Bitmap-Join-Index
-- Als Benutzer HR ausführen
set autotrace on;

select e.first_name, e.last_name, d.department_name, j.job_title
  from employees e
  join departments d on e.department_id = d.department_id
  join jobs j on e.job_id = j.job_id;
   
create bitmap index bix_emp_dept_job
    on employees (d.department_name, j.job_title)
  from employees e, departments d, jobs j
 where e.department_id = d.department_id
   and e.job_id = j.job_id;
   
-- Abfrage (um den Index zu benutzen, muss der (doppelte) Index auf e.department_id entfernt werden
drop index emp_department_ix;
drop index emp_job_ix;

select e.first_name, e.last_name, d.department_name, j.job_title
  from employees e
  join departments d on e.department_id = d.department_id
  join jobs j on e.job_id = j.job_id
 where d.department_name = 'Administration';
 
-- Ausgangssituation wieder herstellen
create index emp_department_ix on employees(department_id);
create index emp_job_ix on employees(job_id);
drop index bix_emp_dept_job;
