-- Umwandlung des Datenmodells SCOTT in ein historisierendes Modell

create table department( 
  dept_id number,
  constraint pk_deptartment primary key(dept_id)
) organization index;

create table department_ref( 
  depr_dept_id number not null, 
  dept_name varchar2(14), 
  dept_location varchar2(13), 
  depr_valid_from date, 
  depr_valid_to date not null,
  constraint pk_department_ref primary key(depr_dept_id, depr_valid_from),
  constraint fk_depr_dept_id foreign key (depr_dept_id)
    references department(dept_id)
);

create index idx_fk_depr_dept_id on department_ref(depr_dept_id);

create table job(
  job_id number,
  constraint pk_job primary key(job_id)
) organization index;

create table job_ref(
  jobr_job_id number not null,
  job_name varchar2(20) not null,
  jobr_valid_from date,
  jobr_valid_to date not null,
  constraint pk_job_ref primary key(jobr_job_id, jobr_valid_from),
  constraint fk_jobr_job_id foreign key(jobr_job_id)
    references job(job_id)
);

create index idx_fk_jobr_job_id on job_ref(jobr_job_id);

create table employee( 
  empl_id number,
  constraint pk_employee primary key(empl_id)
) organization index;


create table employee_ref(
  empr_empl_id number not null,
  empl_name varchar2(14) not null,
  empl_hire_date date, 
  empl_salary number (7,2), 
  empl_commission number (7,2), 
  empr_valid_from date, 
  empr_valid_to date not null,
  constraint pk_employee_ref primary key(empr_empl_id, empr_valid_from),
  constraint fk_empr_empl_id foreign key(empr_empl_id)
    references employee(empl_id)
);

create index idx_fk_empr_empl_id on employee_ref(empr_empl_id);


create table empl_2_mgr(
  emmg_empl_id number,
  emmg_mgr_id number,
  emmg_valid_from date,
  emmg_valid_to date not null,
  constraint pk_empl_2_mgr primary key(emmg_empl_id, emmg_mgr_id, emmg_valid_from),
  constraint fk_emmg_empl_id foreign key(emmg_empl_id)
    references employee(empl_id),
  constraint fk_emmg_mgr_id foreign key(emmg_mgr_id)
    references employee(empl_id)
) organization index;

create index fk_emmg_mgr_id on empl_2_mgr(emmg_mgr_id, emmg_empl_id, emmg_valid_from);

create table empl_2_dept(
  emde_empl_id number,
  emde_dept_id number,
  emde_valid_from date,
  emde_valid_to date,
  constraint pk_empl_2_dept primary key(emde_empl_id, emde_dept_id, emde_valid_from),
  constraint fk_emde_empl_id foreign key(emde_empl_id)
    references employee(empl_id),
  constraint fk_emde_dept_id foreign key(emde_dept_id)
    references department(dept_id)
) organization index;

create index idx_fk_emde_dept_id on empl_2_dept(emde_dept_id, emde_empl_id, emde_valid_from);

create table empl_2_job(
  emjo_empl_id number,
  emjo_job_id number,
  emjo_valid_from date,
  emjo_valid_to date,
  constraint pk_empl_2_job primary key(emjo_empl_id, emjo_job_id, emjo_valid_from),
  constraint fk_emjo_empl_id foreign key(emjo_empl_id)
    references employee(empl_id),
  constraint fk_emjo_job_id foreign key(emjo_job_id)
    references job(job_id)
) organization index;

create index idx_fk_emjo_job_id on empl_2_job(emjo_job_id, emjo_empl_id, emjo_valid_from);

-- Basisdaten aus SCOTT uebernehmen
insert into department (dept_id)
select deptno
  from dept;
commit;

insert into department_ref (depr_dept_id, dept_name, dept_location, depr_valid_from, depr_valid_to)
select deptno, dname, loc, date '1980-01-01', date '2999-12-31'
  from dept;
commit;


insert into job (job_id)
select rownum from dual connect by rownum <= 5;
commit;

insert into job_ref (jobr_job_id, job_name, jobr_valid_from, jobr_valid_to)
select 1, 'CLERK', date '1980-01-01', date '2999-12-31' from dual
union all
select 2, 'SALESMAN', date '1980-01-01', date '2999-12-31' from dual
union all
select 3, 'PRESIDENT', date '1980-01-01', date '2999-12-31' from dual
union all
select 4, 'MANAGER', date '1980-01-01', date '2999-12-31' from dual
union all
select 5, 'ANALYST', date '1980-01-01', date '2999-12-31' from dual;
commit;

insert into employee (empl_id)
select empno
  from emp;
commit;

insert into employee_ref (empr_empl_id, empl_name, empl_hire_date, empl_salary, empl_commission, empr_valid_from, empr_valid_to)
select empno, ename, hiredate, sal, comm, hiredate, date '2999-12-31'
  from emp;
commit;

insert into empl_2_mgr(emmg_empl_id, emmg_mgr_id, emmg_valid_from, emmg_valid_to)
select empno, coalesce(mgr, empno), hiredate, date '2999-12-31'
  from emp;
commit;

insert into empl_2_job (emjo_empl_id, emjo_job_id, emjo_valid_from, emjo_valid_to)
select e.empno, j.job_id, e.hiredate, date '2999-12-31'
  from emp e
  join (select j.job_id, jr.job_name
          from job j
          join job_ref jr on j.job_id = jr.jobr_job_id) j
    on e.job = j.job_name;
commit;

insert into empl_2_dept (emde_empl_id, emde_dept_id, emde_valid_from, emde_valid_to)
select empno, deptno, hiredate, date '2999-12-31'
  from emp;
commit;

-- Abfrage der aktuell gueltigen Daten
select e.empl_id empno,
       er.empl_name ename,
       jr.job_name job,
       em.emmg_mgr_id mgr,
       er.empl_hire_date hiredate,
       er.empl_salary sal,
       er.empl_commission comm,
       ed.emde_dept_id deptno
  from employee e
  join employee_ref er 
    on e.empl_id = er.empr_empl_id
   and sysdate between er.empr_valid_from and er.empr_valid_to
  join empl_2_mgr em
    on e.empl_id = em.emmg_empl_id
   and sysdate between em.emmg_valid_from and em.emmg_valid_to
  join empl_2_job ej
    on e.empl_id = ej.emjo_empl_id
   and sysdate between ej.emjo_valid_from and ej.emjo_valid_to
  join job j
    on ej.emjo_job_id = j.job_id
  join job_ref jr
    on j.job_id = jr.jobr_job_id
   and sysdate between jr.jobr_valid_from and jr.jobr_valid_to
  join empl_2_dept ed
    on e.empl_id = ed.emde_empl_id
   and sysdate between ed.emde_valid_from and ed.emde_valid_to;
