
/* Weitere Hierarchiekonzepte */
-- Auslagerung der Hierarchie in eine eigene Tabelle
create table employee as
select empno, ename, job, sal, comm, hiredate, deptno
  from emp;
  
create table employee_hierarchy as
select empno, nvl(mgr, empno) mgr, 
       hiredate valid_from, to_date('31.12.2099', 'dd.mm.yyyy') valid_to
  from emp;
  
alter table employee add
constraint pk_employee primary key (empno);

alter table employee_hierarchy add
constraint fk_emp_hier_emp_empno
  foreign key (empno) references employee(empno)
  on delete cascade;
  
alter table employee_hierarchy add
constraint fk_emp_hier_emp_mgr
  foreign key (mgr) references employee(empno)
  on delete cascade;
  
alter table employee_hierarchy add
constraint pk_emp_hier primary key (empno, mgr, valid_from);

-- Einfache Connect-By-Abfrage
 select level,
        lpad('.', (level - 1) * 2, '.') || e.ename name
   from employee e
   join employee_hierarchy h on e.empno = h.empno
  start with h.mgr = h.empno
connect by nocycle prior h.empno = h.mgr;

-- Änderung der Daten, um mehrere Manager zu ermöglichen und eine
-- Änderung über die Zeit zu demonstrieren
update employee_hierarchy
   set valid_to = to_date('15.05.2010', 'dd.mm.yyyy') -
                    interval '1' second
 where empno = 7499;
 
insert into employee_hierarchy(empno, mgr, valid_from, valid_to)
values (7499, 7839, 
        to_date('15.05.2010', 'dd.mm.yyyy'), 
        to_date('31.12.2099', 'dd.mm.yyyy'));
        
insert into employee_hierarchy(empno, mgr, valid_from, valid_to)
select empno, 7839, valid_from, valid_to
  from employee_hierarchy
 where empno = 7844;
 
-- Abfrage des neuen Datenbestands
 select level, h.empno, 
        lpad('.', (level - 1) * 2, '.') || e.ename name
   from employee e
   join employee_hierarchy h on e.empno = h.empno
  where sysdate between valid_from and valid_to
  start with h.mgr = h.empno
connect by nocycle prior h.empno = h.mgr;

-- Abfrage für die Vergangenheit, ALLEN ist wieder BLAKE unterstellt
 select level, h.empno, 
        lpad('.', (level - 1) * 2, '.') || e.ename name
   from employee e
   join employee_hierarchy h on e.empno = h.empno
  where to_date('14.05.2010', 'dd.mm.yyyy') between valid_from and valid_to
  start with h.mgr = h.empno
connect by nocycle prior h.empno = h.mgr;

rollback;



