﻿/* Skript zur Demonstration von Outer Joins.
 * Als Benutzer HR ausführen
 */
 
-- "Handgemachte" Abfrage, um zu erkennen, ob alle Abteilungen
-- auch Mitarbeiter beschaeftigen
-- Zunaechst: Wie viele Abteilungen gibt es?
select department_id
  from departments;
  
-- dann: Wie viele Abteilungen werden verwendet?
select distinc department_id
  from employees;

-- Nun die Abfrage: Zeige alle Abteilungen, auch die, die keine Mitarbeiter haben.
select d.department_name, e.last_name 
  from departments d 
  left join employees e 
    on d.department_id = e.department_id;

-- Erweiterung: Zeige nur die Abteilungen, in denen keine Mitarbeiter arbeiten:
select e.last_name, d.department_name 
  from employees e 
  left join departments d 
    on e.department_id = d.department_id
 where d.department_id is null;

-- Oracle-proprietäre Schreibweise
select e.last_name, d.department_name 
  from employees e, departments d 
 where e.department_id = d.department_id (+) 
   and d.department_id is null;

-- Full Outer Join: Anzeige aller Abteilungen ohne Mitarbeiter
-- und aller Mitarbeiter ohne Abteilung
select e.last_name, d.department_name 
  from employees e 
  full join departments d 
    on e.department_id = d.department_id 
 where e.department_id is null 
    or d.department_id is null;

-- Der Exot: CROSS-Join
select e.last_name, d.department_name 
  from employees e 
 cross join departments d;

-- CROSS-Join in Oracle-proprietärer Schreibweise
select e.last_name, d.department_name 
  from employees e, departments d;

-- Der "Anti"-Join
select e.last_name 
  from employees e 
  left join departments d 
    on e.department_id = d.department_id 
 where d.department_id is null;
 
-- Der "Semi"-Join
select distinct d.department_name 
  from departments d 
  join employees e 
    on d.department_id = e.department_id;

